(*
////////////////////////////////////////////////////////////////////////////////
// DBZIPMEM.PAS : EXPLODING DATABASE MEMO COMPONENT                           //
// Copyright  1996-1999 Steve Flynn. All Rights Reserved                     //
//----------------------------------------------------------------------------//
// By using this source code you agree to be bound by the terms of the        //
// following Software License Agreement. Any breach of this agreement will be //
// prosecuted to the full extent of the law.                                  //
////////////////////////////////////////////////////////////////////////////////

LICENSE AGREEMENT - MARCH 1999

USE OF THE SOFTWARE

Steven M. Flynn (the "Author") grants you, the end user, a non-exclusive
single-user license to use the supplied software program and all associated
materials including the Source Code (the "Software"). The Software and all
Intellectual Property embodied in the Software always remains the property of
the Author. Your use of the Software indicates your acceptance in full of the
terms and conditions of this agreement.


COPYING THE SOFTWARE

You may freely copy the Software provided that such copies do not infringe upon
any other part of this agreement.


DISTRIBUTION OF THE SOFTWARE

The Software may be re-distributed royalty free providing that it is not
modified in any way and that no charge is made for the Software itself. Only a
nominal fee for media and/or copying services is allowed. All files, including
the License Agreement, that are supplied by the Author with the Software must
also be included in any such distribution.


DERIVED WORKS

You may NOT use the Software to derive any works which are based substantially
in part or in full on the Software or any portion of the Software. Any such
derived works will remain the property of the Author.


MODIFIED VERSIONS OF THE SOFTWARE

All updated, revised, modified or improved versions of the Software are also
subject to the conditions of this license agreement and shall remain the
property of the Author. You are required under the terms of this agreement to
submit to the Author any and all improvements or additions made by you to the 
Software for future inclusion in the general distribution of the Software by
the Author.


TERMINATION OF THIS AGREEMENT

You may terminate this License Agreement at any time by destroying all copies
of the Software in your possession.


LIMITED WARRANTY

The Software is distributed and licensed "AS IS". The Author specifically
disclaims all other warranties, express or implied, including but not limited
to, implied warranties of merchantability and fitness for a particular purpose,
with regard to the Software.


DAMAGES

By using the Software you do so at your own risk. In no event shall the Author
be responsible for any damages whatsoever (including but not limited to,
damages for loss of business profits, business interruption, loss of business
information, or any other pecuniary loss or any other real or consequential 
damages) arising out of the use or inability to use this product.


LIABILITY

In the event of failure of the Software, for any reason, the Author's sole
liability shall be to refund to you the amount paid by you to the Author for
the use of the Software.


COPYRIGHT

The Software and the Intellectual Property embodied in the Software shall
always remain the property of the Author and is protected by Australian
Copyright Law and International Treaty Provisions.


AGREEMENT

Your use of the Software indicates your agreement with the above terms and
conditions. If you do not agree with these terms and conditions then you must
destroy all copies of the software in your possession.


CONSUMER RIGHTS

If any clause in the above Software License Agreement is in violation of your
rights under your local laws then you must cease using the Software
immediately. It is your own responsibility to be aware of your legal rights
under your own local laws.


END OF SOFTWARE LICENSE AGREEMENT
*)

{$A-,B-,D-,F-,G+,I-,L-,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y-,Z-}
{$IFDEF WIN32}
{$H+,J+}
{$M 16384,1048576}
{$IMAGEBASE $00400000}
{$R DBZIPMEM.R32}
{$ELSE}
{$K-}
{$M 16384,8192}
{$R DBZIPMEM.R16}
{$ENDIF}
{$C PRELOAD}

unit DBZipMem;

interface

uses
	SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls, Dialogs,
	StdCtrls, Buttons, DBCtrls, ExtCtrls, Forms;

type
	TDBZipMemo = class;
	TFudge = class(TWinControl);

{// TZipForm  /////////////////////////////////////////////////////////////////}
	TZipForm = class(TForm)
		MemoPanel: TPanel;
		Memo: TMemo;
		StatusPanel: TPanel;
		HintPanel: TPanel;
		ButtonHolder: TPanel;
		OKButton: TBitBtn;
		CancelButton: TBitBtn;
		procedure OKButtonClick(Sender: TObject);
		procedure CancelButtonClick(Sender: TObject);

	private
		MasterMemo: TDBZipMemo;
		procedure CloseForm(Sender: TObject; var CanClose: Boolean);

	protected
		procedure CreateParams(var Params: TCreateParams); override;

	public
		constructor Create(AOwner: TComponent); override;
		end;


{// TDBZipMemo ////////////////////////////////////////////////////////////////}
	TDBZipMemo = class(TDBMemo)
	private
		FAlreadyOpen: boolean;
		FAutoTab: boolean;
		FZipForm: TZipForm;
		FScrollBars: TScrollStyle;

	protected
		procedure OpenMemoWindow;
		procedure CloseMemoWindow;
		procedure Loaded; override;
		procedure DblClick; override;

	public
		constructor Create(AOwner: TComponent); override;

	published
		property AutoTab: boolean read FAutoTab write FAutoTab default False;
	end;

procedure Register;

implementation

{$R *.DFM}

{// Control Registration //////////////////////////////////////////////////////}
procedure Register;
	begin
	RegisterComponents('Data Controls', [TDBZipMemo]);
	end;


{// TDBZipMemo ////////////////////////////////////////////////////////////////}
constructor TDBZipMemo.Create(AOwner: TComponent);
	begin
	inherited Create(AOwner);
	FAlreadyOpen := False;
	FAutoTab := False;
	end;


procedure TDBZipMemo.Loaded;
	begin
	inherited Loaded;
	FScrollBars := ScrollBars;
	if (csDesigning in ComponentState) then
		exit;
	ScrollBars := ssNone;
	end;


procedure TDBZipMemo.OpenMemoWindow;
	var
	i: integer;
	s: string;
	begin
	if FAlreadyOpen then
		exit;
	FAlreadyOpen := True;
	s := Hint;
	for i := 1 to Length(s) do
		if s[i] = #13 then
			s[i] := ' ';
	Application.CreateForm(TZipForm, FZipForm);
	with FZipForm do
		begin
		FormStyle := fsStayOnTop;
		ActiveControl := Memo;
		KeyPreview := True;
		Memo.Text := self.Text;
		Memo.ScrollBars := FScrollBars;
		Memo.Color := self.Color;
		Memo.Font := self.Font;
		Memo.WordWrap := self.WordWrap;
		Memo.Hint := self.Hint;
		Caption := GetLongHint(s);
		HintPanel.Caption := GetLongHint(s);
		MasterMemo := self;
		Visible := True;
		end;
	end;


procedure TDBZipMemo.CloseMemoWindow;
	begin
	SelStart := 0;
	SelLength := 0;
	FAlreadyOpen := False;
	end;


procedure TDBZipMemo.DblClick;
	begin
	inherited DblClick;
	OpenMemoWindow;
	end;


{// TZipForm //////////////////////////////////////////////////////////////////}
constructor TZipForm.Create(AOwner: TComponent);
	begin
	inherited Create(AOwner);
	OnCloseQuery := CloseForm;
	end;


procedure TZipForm.CreateParams(var Params: TCreateParams);
	begin
	inherited CreateParams(Params);
	Params.WindowClass.Style := CS_SAVEBITS;
	end;


procedure TZipForm.CloseForm(Sender: TObject; var CanClose: Boolean);
	var
	MParent: TFudge;
	begin
	MParent := TFudge(MasterMemo.Parent);
	MasterMemo.FZipForm := nil;
	if MasterMemo.FAutoTab then
		MParent.SelectNext(MasterMemo, True, True);
	MasterMemo.CloseMemoWindow;
	CanClose := True;
	end;


procedure TZipForm.OKButtonClick(Sender: TObject);
	begin
	MasterMemo.DataSource.Edit;
	MasterMemo.Text := Memo.Text;
	MasterMemo.Modified := True;
	Close;
	end;


procedure TZipForm.CancelButtonClick(Sender: TObject);
	begin
	Close;
	end;

end.
